/**
 * Author = Robbert Harms
 * Date = 2014-09-21
 * License = LGPL v3
 * Maintainer = Robbert Harms
 * Email = robbert.harms@maastrichtuniversity.nl
 */

/**
 * A stub for consistency. See MRIConstants.h for the definitions.
 */
